@@ -1,5 +1,5 @@
 /*
- * Copyright 2016 Advanced Micro Devices, Inc.
+ * Copyright 2023 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,10 +20,9 @@
  * OTHER DEALINGS IN THE SOFTWARE.
  *
  */
-#ifndef __MMHUB_V1_7_H__
-#define __MMHUB_V1_7_H__
+#ifndef __MMHUB_V4_1_0_H__
+#define __MMHUB_V4_1_0_H__
 
-extern const struct amdgpu_mmhub_funcs mmhub_v1_7_funcs;
-extern struct amdgpu_mmhub_ras mmhub_v1_7_ras;
+extern const struct amdgpu_mmhub_funcs mmhub_v4_1_0_funcs;
 
 #endif
